@@ -19,15 +19,8 @@ debugsym=None
 # C parser analysis code
 #
 ignored_files = {
-  "trio": "too many non standard macros",
-  "trio.c": "too many non standard macros",
-  "trionan.c": "too many non standard macros",
-  "triostr.c": "too many non standard macros",
   "config.h": "generated portability layer",
   "libxml.h": "internal only",
-  "testOOM.c": "out of memory tester",
-  "testOOMlib.h": "out of memory tester",
-  "testOOMlib.c": "out of memory tester",
   "rngparser.c": "not yet integrated",
   "testModule.c": "test tool",
   "testThreads.c": "test tool",
@@ -63,7 +56,14 @@ ignored_words = {
   "LIBXML_ATTR_FORMAT": (5, "macro for gcc printf args checking extension"),
   "LIBXML_ATTR_ALLOC_SIZE": (3, "macro for gcc checking extension"),
   "ATTRIBUTE_NO_SANITIZE": (3, "macro keyword"),
+  "ATTRIBUTE_NO_SANITIZE_INTEGER": (0, "macro keyword"),
   "XML_DEPRECATED": (0, "macro keyword"),
+  "XML_GLOBALS_ALLOC": (0, "macro keyword"),
+  "XML_GLOBALS_ERROR": (0, "macro keyword"),
+  "XML_GLOBALS_IO": (0, "macro keyword"),
+  "XML_GLOBALS_PARSER": (0, "macro keyword"),
+  "XML_GLOBALS_TREE": (0, "macro keyword"),
+  "XML_THREAD_LOCAL": (0, "macro keyword"),
 }
 
 def escape(raw):
@@ -292,7 +292,7 @@ class index:
                  continue
              if id in self.enums:
                  continue
-             if id in self.macros:
+             if id in self.macros and id != 'XML_OP':
                  print("macro %s from %s redeclared in %s" % (
                     id, self.macros[id].header, idx.macros[id].header))
              else:
@@ -905,7 +905,7 @@ class CParser:
                 i = i + 1
             if retdesc == "" and ret[0] != "void":
                 self.warning("Function comment for %s lacks description of return value" % (name))
-            if desc == "":
+            if desc == "" and retdesc == "":
                 self.warning("Function comment for %s lacks description of the function" % (name))
 
         return(((ret[0], retdesc), args, desc))
@@ -1264,26 +1264,29 @@ class CParser:
         if token == None:
             return token
 
+        have_sign = 0
+        done = 0
+
         while token[0] == "name" and (
               token[1] == "const" or \
               token[1] == "unsigned" or \
               token[1] == "signed"):
+            if token[1] == "unsigned" or token[1] == "signed":
+                have_sign = 1
             if self.type == "":
                 self.type = token[1]
             else:
                 self.type = self.type + " " + token[1]
             token = self.token()
 
-        if token[0] == "name" and (token[1] == "long" or token[1] == "short"):
+        if token[0] == "name" and token[1] in ("char", "short", "int", "long"):
             if self.type == "":
                 self.type = token[1]
             else:
                 self.type = self.type + " " + token[1]
-            if token[0] == "name" and token[1] == "int":
-                if self.type == "":
-                    self.type = tmp[1]
-                else:
-                    self.type = self.type + " " + tmp[1]
+
+        elif have_sign:
+            done = 1
 
         elif token[0] == "name" and token[1] == "struct":
             if self.type == "":
@@ -1352,7 +1355,8 @@ class CParser:
             self.error("parsing type %s: expecting a name" % (self.type),
                        token)
             return token
-        token = self.token()
+        if not done:
+            token = self.token()
         while token != None and (token[0] == "op" or
               token[0] == "name" and token[1] == "const"):
             self.type = self.type + " " + token[1]
@@ -1770,7 +1774,8 @@ class docBuilder:
         try:
             (ret, params, desc) = id.info
             if (desc == None or desc == '') and \
-               name[0:9] != "xmlThrDef" and name != "xmlDllMain":
+               name[0:9] != "xmlThrDef" and name != "xmlDllMain" and \
+               ret[1] == '':
                 print("%s %s from %s has no description" % (id.type, name,
                        self.modulename_file(id.module)))
 
@@ -1804,7 +1809,8 @@ class docBuilder:
                                  escape(dict.info[data]),
                                  data.lower()))
                 except:
-                    print("Header %s lacks a %s description" % (module, data))
+                    if data != 'Author':
+                        print("Header %s lacks a %s description" % (module, data))
             if 'Description' in dict.info:
                 desc = dict.info['Description']
                 if desc.find("DEPRECATED") != -1:
